unit MyAddin1_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, Variants, adxAddIn, MyAddin1_TLB, Classes, StdVcl,
  adxOffice, Excel2000, Word2000, Dialogs, OleServer, adxHostAppEvents, Windows;

type
  TcoMyAddin1 = class(TadxAddin, IcoMyAddin1)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    adxCommandBar1: TadxCommandBar;
    adxExcelAppEvents1: TadxExcelAppEvents;
    adxWordAppEvents1: TadxWordAppEvents;
    adxRibbonTab1: TadxRibbonTab;
    ExcelWorksheet1: TExcelWorksheet;
    procedure adxRibbonTab1Controls0Controls0Controls0Click(Sender: TObject;
      const RibbonControl: IRibbonControl);
    procedure adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
    procedure adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
    procedure adxWordAppEvents1WindowDeactivate(ASender: TObject;
      const Doc: _Document; const Wn: Window);
    procedure adxWordAppEvents1WindowActivate(ASender: TObject;
      const Doc: _Document; const Wn: Window);
    procedure adxExcelAppEvents1WindowDeactivate(ASender: TObject;
      const Wb: _Workbook; const Wn: Window);
    procedure adxExcelAppEvents1WindowActivate(ASender: TObject;
      const Wb: _Workbook; const Wn: Window);
    procedure ExcelWorksheet1BeforeRightClick(ASender: TObject;
      const Target: ExcelRange; var Cancel: WordBool);
    procedure adxCommandBar1Controls0Click(Sender: TObject);
    procedure AddInModuleTaskPanes0VisibleStateChange(Sender: TObject;
      Instance: TadxCustomTaskPaneInstance);
    procedure adxWordAppEvents1WindowSelectionChange(ASender: TObject;
      const Sel: WordSelection);
    procedure adxExcelAppEvents1SheetSelectionChange(ASender: TObject;
      const Sh: IDispatch; const Target: ExcelRange);
    procedure adxExcelAppEvents1SheetActivate(ASender: TObject;
      const Sh: IDispatch);
    procedure adxExcelAppEvents1WorkbookActivate(ASender: TObject;
      const Wb: _Workbook);
    procedure adxCommandBar1Controls1Click(Sender: TObject);
    procedure adxRibbonTab1Controls0Controls0Controls1Click(
      Sender: TObject; const RibbonControl: IRibbonControl);
    procedure adxExcelAppEvents1NewWorkbook(ASender: TObject;
      const Wb: _Workbook);
    procedure adxExcelAppEvents1SheetBeforeRightClick(ASender: TObject;
      const Sh: IDispatch; const Target: ExcelRange; var Cancel: WordBool);
    procedure adxCOMAddInModuleTaskPaneAfterShow(Sender: TadxCustomTaskPane;
      Instance: TadxCustomTaskPaneInstance);
    procedure adxCOMAddInModuleAddInInitialize(Sender: TObject);
  private
    FVersion: string;
    Sheet: ExcelWorksheet;
    function ConnectToSheet(): boolean ;
    function GetInfoString(const Window: IDispatch): string;
    procedure RefreshTaskPane(const Window: IDispatch);
  protected
  public
  end;

implementation

{$R *.dfm}

procedure TAddInModule.adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
begin
  if Assigned(Sheet) then
  begin
    ExcelWorksheet1.Disconnect();
    ExcelWorksheet1.Free();
    Sheet := nil;
  end;
end;

procedure TAddInModule.adxCOMAddInModuleAddInInitialize(Sender: TObject);
begin
  FVersion := '';
  case Self.HostType of
    ohaExcel:
      FVersion := ExcelApp.Version[adxLCID];
    ohaWord:
      FVersion := WordApp.Version;
  end;
  if FVersion <> '' then
    FVersion := Copy(FVersion, 1, 4);
end;

procedure TAddInModule.adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
begin
  case Self.HostType of
    ohaExcel:
      adxCommandBar1.Controls[0].Enabled := ConnectToSheet();
    ohaWord:
      adxCommandBar1.Controls[0].Enabled := true;
    else
      ShowMessage(Self.AddInName +
          ' doesn''t support the current host application');
  end;
end;

procedure TAddInModule.adxCOMAddInModuleTaskPaneAfterShow(
  Sender: TadxCustomTaskPane; Instance: TadxCustomTaskPaneInstance);
begin
  RefreshTaskPane(Instance.Window);
end;

procedure TAddInModule.adxCommandBar1Controls0Click(Sender: TObject);
begin
  case Self.HostType of
    ohaExcel:
      ShowMessage('The current cell is '
          + Self.ExcelApp.ActiveCell.AddressLocal
            [False, False, xlA1, EmptyParam, EmptyParam]);
    ohaWord:
      if Self.WordApp.Selection <> nil then
          ShowMessage('There are '
            + IntToStr(Self.WordApp.Selection.Words.Count)
            + ' words currently selected');
    else
        ShowMessage(Self.AddInName +
          ' doesn''t support the current host application');
    end;
end;

procedure TAddInModule.adxExcelAppEvents1WindowActivate(ASender: TObject; const Wb: _Workbook;
  const Wn: Window);
begin
    adxCommandBar1.Controls[0].Enabled := ConnectToSheet();
    RefreshTaskPane(ExcelApp.ActiveWindow);
end;

procedure TAddInModule.adxExcelAppEvents1WindowDeactivate(ASender: TObject; const Wb: _Workbook;
  const Wn: Window);
begin
    adxCommandBar1.Controls[0].Enabled := false;
    if Assigned(Sheet) then
      ExcelWorksheet1.Disconnect();
    end;

procedure TAddInModule.adxRibbonTab1Controls0Controls0Controls0Click(
  Sender: TObject; const RibbonControl: IRibbonControl);
begin
    adxCommandBar1Controls0Click(nil);
end;

procedure TAddInModule.adxWordAppEvents1WindowActivate(ASender: TObject; const Doc: _Document;
  const Wn: Window);
begin
    adxCommandBar1.Controls[0].Enabled := true;
end;

procedure TAddInModule.adxWordAppEvents1WindowDeactivate(ASender: TObject; const Doc: _Document;
  const Wn: Window);
begin
    adxCommandBar1.Controls[0].Enabled := false;
end;

procedure TAddInModule.ExcelWorksheet1BeforeRightClick(ASender: TObject;
  const Target: ExcelRange; var Cancel: WordBool);
begin
  if Target.Address[False, False, xlA1, EmptyParam, EmptyParam][1] = 'A' then
  begin
    ShowMessage('Context menu will not be shown!');
    Cancel := true;
  end;
end;

function TAddInModule.ConnectToSheet(): boolean;
var
  IUnk: IUnknown;
begin
    IUnk := Self.ExcelApp.ActiveSheet;
    Result := false;
    if Assigned(IUnk) then
    begin
      IUnk.QueryInterface(IID__WorkSheet, Sheet);
      if Assigned(Sheet) then
      begin
        ExcelWorksheet1.ConnectTo(Sheet);
        Result := true;
      end;
    end;
end;

procedure TAddInModule.AddInModuleTaskPanes0VisibleStateChange(
  Sender: TObject; Instance: TadxCustomTaskPaneInstance);
var IControlButton: CommandBarButton;
    RibbonButton: TadxRibbonButton;
    WindowString: string;
begin
  if (not Instance.Visible) then
  begin
    case HostType of
      ohaExcel:
        WindowString := 'an Excel window!';
      ohaWord:
        WindowString := 'a Word window!';
      else
        WindowString := 'an unknown window!';
    end;
    try
      IControlButton :=
        FindControl(msoControlButton,'RestoreMyTaskPane') as CommandBarButton;
      if Assigned(IControlButton) then
        IControlButton.Visible := true;
      RibbonButton := FindRibbonControl('RestoreMyTaskViaRibbon') as TadxRibbonButton;
      if Assigned(RibbonButton) then
        RibbonButton.Visible := true;
    finally
      IControlButton := nil;
    end;
    ShowMessage('You''ve closed the task pane instance in ' + WindowString);
  end;
end;

function TAddInModule.GetInfoString(const Window: IDispatch): string;
var er: ExcelRange;
    ws: Word2000.Window;
begin
  if Window <> nil then
    case HostType of
      ohaExcel:
      try
        er := (Window as Excel2000.Window).ActiveCell;
        if er <> nil then
          Result := 'The current cell is: '
            +  er.AddressLocal[False, False, xlA1, EmptyParam, EmptyParam]; //relative address
      finally
        er := nil;
      end;
      ohaWord:
      try
        ws := Window as Word2000.Window;
        if ws <> nil then
          Result := 'The current selection contains '
                 + IntToStr(ws.Selection.Range.Words.Count)
                 + ' words';
      finally
        ws := nil;
      end;
    else
      Result :=  'The ' + AddinName + ' COM Add-in doesn''t support the current host application!' ;
    end;
end;

procedure TAddInModule.RefreshTaskPane(const Window: IDispatch);
var
  ITaskPane: IMyTaskPane;
  TaskPaneInstance: TadxCustomTaskPaneInstance;
begin
  if (FVersion = '12.0') and (Window <> nil) and (TaskPanes.Count > 0) then
  begin
    TaskPaneInstance := TaskPanes.Items[0].Instances.Items[Window];
    if Assigned(TaskPaneInstance) then
    begin
      TaskPaneInstance.Control.QueryInterface(IMyTaskPane, ITaskPane);
      if Assigned(ITaskPane) then
      try
        ITaskPane.InfoString := GetInfoString(Window);
      finally
        ITaskPane := nil;
      end;
    end;
  end;
end;

procedure TAddInModule.adxWordAppEvents1WindowSelectionChange(
  ASender: TObject; const Sel: WordSelection);
begin
  if Assigned(Sel) then
    RefreshTaskPane(WordApp.ActiveWindow);
end;

procedure TAddInModule.adxExcelAppEvents1SheetSelectionChange(
  ASender: TObject; const Sh: IDispatch; const Target: ExcelRange);
begin
  RefreshTaskPane(ExcelApp.ActiveWindow);
end;

procedure TAddInModule.adxExcelAppEvents1NewWorkbook(ASender: TObject;
  const Wb: _Workbook);
begin
  RefreshTaskPane(ExcelApp.ActiveWindow);
end;

procedure TAddInModule.adxExcelAppEvents1SheetActivate(ASender: TObject;
  const Sh: IDispatch);
begin
  RefreshTaskPane(ExcelApp.ActiveWindow);
end;

procedure TAddInModule.adxExcelAppEvents1SheetBeforeRightClick(ASender: TObject;
  const Sh: IDispatch; const Target: ExcelRange; var Cancel: WordBool);
begin
  if Target.Address[False, False, xlA1, EmptyParam, EmptyParam][1] = 'A' then
  begin
    ShowMessage('Context menu will not be shown!');
    Cancel := true;
  end;
end;

procedure TAddInModule.adxExcelAppEvents1WorkbookActivate(ASender: TObject;
  const Wb: _Workbook);
begin
  RefreshTaskPane(ExcelApp.ActiveWindow);
end;

procedure TAddInModule.adxCommandBar1Controls1Click(Sender: TObject);
var
  Window: IDispatch;
  TaskPaneInstance: TadxCustomTaskPaneInstance;
  IControlButton: CommandBarButton;
  RibbonButton: TadxRibbonButton;
begin
  case HostType of
    ohaExcel:
      Window := ExcelApp.ActiveWindow;
    ohaWord:
      Window := WordApp.ActiveWindow;
    else
      Window := nil;
  end;
  if (Window <> nil) and
   (TaskPanes.Count > 0) and (TaskPanes.Items[0].Instances.Count > 0) then
  begin
    TaskPaneInstance := TaskPanes.Items[0].Instances.Items[Window];
    if Assigned(TaskPaneInstance) then
      TaskPaneInstance.Visible := true;
    if Sender is TadxCommandBarButton  then
    begin
      TadxCommandBarButton(Sender).Visible := false;
      RibbonButton := FindRibbonControl('RestoreMyTaskPaneViaRibbon') as TadxRibbonButton;
      if Assigned(RibbonButton) then
        RibbonButton.Visible := false;
    end
    else
    begin
      if Sender is TadxRibbonButton then
      begin
        TadxRibbonButton(Sender).Visible := false;
        try
          IControlButton :=
            FindControl(msoControlButton,'RestoreMyTaskPane') as CommandBarButton;
          if Assigned(IControlButton) then
            IControlButton.Visible := false;
        finally
          IControlButton := nil;
        end;
      end;
    end;
  end;
end;

procedure TAddInModule.adxRibbonTab1Controls0Controls0Controls1Click(
  Sender: TObject; const RibbonControl: IRibbonControl);
begin
  adxCommandBar1Controls1Click(Sender)
end;

initialization
  TadxFactory.Create(ComServer, TcoMyAddin1, CLASS_coMyAddin1, TAddInModule);

end.
